@@ -2,7 +2,7 @@
  *
  * job_process.c - job process handling
  *
- * Copyright © 2010 Canonical Ltd.
+ * Copyright © 2011 Canonical Ltd.
  * Author: Scott James Remnant <scott@netsplit.com>.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -38,6 +38,9 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#include <utmp.h>
+#include <utmpx.h>
+#include <pwd.h>
 
 #include <nih/macros.h>
 #include <nih/alloc.h>
@@ -385,6 +388,11 @@ job_process_spawn (JobClass     *class,
 	 */
 	pid = fork ();
 	if (pid > 0) {
+		if (class->debug) {
+			nih_info (_("Pausing %s (%d) [pre-exec] for debug"),
+			  class->name, pid);
+		}
+
 		sigprocmask (SIG_SETMASK, &orig_set, NULL);
 		close (fds[1]);
 
@@ -430,8 +438,20 @@ job_process_spawn (JobClass     *class,
 	 * the FD_CLOEXEC flag so it's automatically closed when we exec()
 	 * later.
 	 */
-	if (system_setup_console (class->console, FALSE) < 0)
-		job_process_error_abort (fds[1], JOB_PROCESS_ERROR_CONSOLE, 0);
+	if (system_setup_console (class->console, FALSE) < 0) {
+		if (class->console == CONSOLE_OUTPUT) {
+			NihError *err;
+
+			err = nih_error_get ();
+			nih_warn (_("Failed to open system console: %s"),
+				  err->message);
+			nih_free (err);
+
+			if (system_setup_console (CONSOLE_NONE, FALSE) < 0)
+				job_process_error_abort (fds[1], JOB_PROCESS_ERROR_CONSOLE, 0);
+		} else
+			job_process_error_abort (fds[1], JOB_PROCESS_ERROR_CONSOLE, 0);
+	}
 
 	/* Set resource limits for the process, skipping over any that
 	 * aren't set in the job class such that they inherit from
@@ -484,6 +504,16 @@ job_process_spawn (JobClass     *class,
 		}
 	}
 
+	/* Handle changing a chroot session job prior to dealing with
+	 * the 'chroot' stanza.
+	 */
+	if (class->session && class->session->chroot) {
+		if (chroot (class->session->chroot) < 0) {
+			nih_error_raise_system ();
+			job_process_error_abort (fds[1], JOB_PROCESS_ERROR_CHROOT, 0);
+		}
+	}
+
 	/* Change the root directory, confining path resolution within it;
 	 * we do this before the working directory call so that is always
 	 * relative to the new root.
@@ -513,6 +543,23 @@ job_process_spawn (JobClass     *class,
 	nih_signal_reset ();
 	sigprocmask (SIG_SETMASK, &orig_set, NULL);
 
+	/* Notes:
+	 *
+	 * - we can't use pause() here since there would then be no way to
+	 *   resume the process without killing it.
+	 *
+	 * - we have to close the pipe back to the parent since if we don't,
+	 *   the parent hangs until the STOP is cleared. Although this may be
+	 *   acceptable for normal operation, this causes the test suite to
+	 *   fail. Note that closing the pipe means from this point onwards,
+	 *   the parent cannot know the true outcome of the spawn: that
+	 *   responsibility lies with the debugger.
+	 */
+	if (class->debug) {
+		close (fds[1]);
+		raise (SIGSTOP);
+	}
+
 	/* Set up a process trace if we need to trace forks */
 	if (trace) {
 		if (ptrace (PTRACE_TRACEME, 0, NULL, 0) < 0) {
@@ -522,6 +569,54 @@ job_process_spawn (JobClass     *class,
 		}
 	}
 
+	/* Handle unprivileged user job by dropping privileges to
+	 * their level.
+	 */
+	if (class->session && class->session->user) {
+		uid_t uid = class->session->user;
+		struct passwd *pw = NULL;
+
+		/* D-Bus does not expose a public API call to allow
+		 * us to query a users primary group.
+		 * _dbus_user_info_fill_uid () seems to exist for this
+		 * purpose, but is a "secret" API. It is unclear why
+		 * D-Bus neglects the gid when it allows the uid
+		 * to be queried directly.
+		 *
+		 * Our only recourse is to disallow user sessions in a
+		 * chroot and assume that all other user sessions
+		 * originate from the local system. In this way, we can
+		 * bypass D-Bus and use getpwuid ().
+		 */
+
+		if (class->session->chroot) {
+			/* We cannot determine the group id of the user
+			 * session in the chroot via D-Bus, so disallow
+			 * all jobs in such an environment.
+			 */
+			_nih_error_raise (__FILE__, __LINE__, __FUNCTION__,
+					EPERM, strerror (EPERM));
+		}
+
+		pw = getpwuid (uid);
+
+		if (!pw)
+			nih_return_system_error (-1);
+
+		nih_assert (pw->pw_uid == uid);
+
+		if (uid && setuid (uid) < 0) {
+			nih_error_raise_system ();
+			job_process_error_abort (fds[1], JOB_PROCESS_ERROR_SETUID, 0);
+		}
+
+		if (pw->pw_gid && setgid (pw->pw_gid) < 0) {
+			nih_error_raise_system ();
+			job_process_error_abort (fds[1], JOB_PROCESS_ERROR_SETGID, 0);
+		}
+	}
+
+
 	/* Execute the process, if we escape from here it failed */
 	if (execvp (argv[0], argv) < 0) {
 		nih_error_raise_system ();
@@ -965,6 +1060,8 @@ job_process_terminated (Job         *job,
 			int          status)
 {
 	int failed = FALSE, stop = FALSE, state = TRUE;
+	struct utmpx *utmptr;
+	struct timeval tv;
 
 	nih_assert (job != NULL);
 
@@ -1127,6 +1224,32 @@ job_process_terminated (Job         *job,
 		job->kill_process = -1;
 	}
 
+	/* Find existing utmp entry for the process pid */
+	setutxent();
+	while ((utmptr = getutxent()) != NULL) {
+		if (utmptr->ut_pid == job->pid[process]) {
+			/* set type and clean ut_user, ut_host,
+			 * ut_time as described in utmp(5)
+			 */
+			utmptr->ut_type = DEAD_PROCESS;
+			memset(utmptr->ut_user, 0, UT_NAMESIZE);
+			memset(utmptr->ut_host, 0, UT_HOSTSIZE);
+			utmptr->ut_time = 0;
+			/* Update existing utmp file. */
+			pututxline(utmptr);
+
+			/* set ut_time for log */
+			gettimeofday(&tv, NULL);
+			utmptr->ut_tv.tv_sec = tv.tv_sec;
+			utmptr->ut_tv.tv_usec = tv.tv_usec;
+			/* Write wtmp entry */
+			updwtmpx (_PATH_WTMP, utmptr);
+
+			break;
+		}
+	}
+	endutxent();
+
 	/* Clear the process pid field */
 	job->pid[process] = 0;
 
