@@ -2,7 +2,7 @@
  *
  * parse_job.c - job definition parsing
  *
- * Copyright © 2009 Canonical Ltd.
+ * Copyright © 2010 Canonical Ltd.
  * Author: Scott James Remnant <scott@netsplit.com>.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -209,6 +209,14 @@ static int stanza_chdir       (JobClass *class, NihConfigStanza *stanza,
 			       const char *file, size_t len,
 			       size_t *pos, size_t *lineno)
 	__attribute__ ((warn_unused_result));
+static int stanza_debug       (JobClass *class, NihConfigStanza *stanza,
+			       const char *file, size_t len,
+			       size_t *pos, size_t *lineno)
+	__attribute__ ((warn_unused_result));
+static int stanza_manual      (JobClass *class, NihConfigStanza *stanza,
+			       const char *file, size_t len,
+			       size_t *pos, size_t *lineno)
+	__attribute__ ((warn_unused_result));
 
 
 /**
@@ -245,6 +253,8 @@ static NihConfigStanza stanzas[] = {
 	{ "limit",       (NihConfigHandler)stanza_limit       },
 	{ "chroot",      (NihConfigHandler)stanza_chroot      },
 	{ "chdir",       (NihConfigHandler)stanza_chdir       },
+	{ "debug",       (NihConfigHandler)stanza_debug       },
+	{ "manual",      (NihConfigHandler)stanza_manual      },
 
 	NIH_CONFIG_LAST
 };
@@ -253,6 +263,8 @@ static NihConfigStanza stanzas[] = {
 /**
  * parse_job:
  * @parent: parent object for new job,
+ * @session: session,
+ * @update: If not NULL, update the existing specified JobClass,
  * @name: name of new job,
  * @file: file or string to parse,
  * @len: length of @file,
@@ -268,10 +280,13 @@ static NihConfigStanza stanzas[] = {
  * of the returned job are freed, the returned job will also be
  * freed.
  *
- * Returns: new JobClass structure on success, NULL on raised error.
+ * Returns: If @update is NULL, returns new JobClass structure on success, NULL on raised error.
+ * If @update is not NULL, returns @update on success, or NULL on error.
  **/
 JobClass *
 parse_job (const void *parent,
+	   Session *   session,
+	   JobClass   *update,
 	   const char *name,
 	   const char *file,
 	   size_t      len,
@@ -284,13 +299,22 @@ parse_job (const void *parent,
 	nih_assert (file != NULL);
 	nih_assert (pos != NULL);
 
-	class = job_class_new (parent, name);
-	if (! class)
-		nih_return_system_error (NULL);
+	if (update) {
+		class = update;
+		nih_debug ("Reusing JobClass %s (%s)",
+				class->name, class->path);
+	} else {
+	  nih_debug ("Creating new JobClass %s",
+			  name);
+	  class = job_class_new (parent, name, session);
+	  if (! class)
+		  nih_return_system_error (NULL);
+	}
 
 	if (nih_config_parse_file (file, len, pos, lineno,
-				   stanzas, class) < 0) {
-		nih_free (class);
+				stanzas, class) < 0) {
+		if (!update)
+			nih_free (class);
 		return NULL;
 	}
 
@@ -964,6 +988,77 @@ parse_on_collect (JobClass       *class,
 }
 
 
+/**
+ * stanza_debug:
+ * @class: job class being parsed,
+ * @stanza: stanza found,
+ * @file: file or string to parse,
+ * @len: length of @file,
+ * @pos: offset within @file,
+ * @lineno: line number.
+ *
+ * Parse a debug stanza from @file. No parameters are supported.
+ *
+ * Returns: zero on success, negative value on error.
+ **/
+static int
+stanza_debug (JobClass           *class,
+		 NihConfigStanza *stanza,
+		 const char      *file,
+		 size_t           len,
+		 size_t          *pos,
+		 size_t          *lineno)
+{
+	nih_assert (class != NULL);
+	nih_assert (stanza != NULL);
+	nih_assert (file != NULL);
+	nih_assert (pos != NULL);
+
+	class->debug = TRUE;
+
+	return 0;
+}
+
+
+/**
+ * stanza_manual:
+ * @class: job class being parsed,
+ * @stanza: stanza found,
+ * @file: file or string to parse,
+ * @len: length of @file,
+ * @pos: offset within @file,
+ * @lineno: line number.
+ *
+ * Parse a manual stanza from @file. No parameters are supported.
+ *
+ * Returns: zero on success, negative value on error.
+ **/
+static int
+stanza_manual (JobClass           *class,
+		 NihConfigStanza *stanza,
+		 const char      *file,
+		 size_t           len,
+		 size_t          *pos,
+		 size_t          *lineno)
+{
+	nih_assert (class != NULL);
+	nih_assert (stanza != NULL);
+	nih_assert (file != NULL);
+	nih_assert (pos != NULL);
+
+	/* manual simply disregards any start on events seen previously */
+
+	nih_debug ("disregarding start on events for %s",
+			class->name);
+
+	if (class->start_on)
+		nih_unref (class->start_on, class);
+
+	class->start_on = NULL;
+
+	return 0;
+}
+
 /**
  * stanza_instance:
  * @class: job class being parsed,
@@ -1216,8 +1311,8 @@ stanza_export (JobClass        *class,
  * @lineno: line number.
  *
  * Parse a start stanza from @file.  This stanza expects a second "on"
- * argument, followed by an event which is allocated as an EventInfo structure
- * and stored in the start events list of the class.
+ * argument, followed by an event which is allocated as an EventOperator
+ * structure and stored in the start events list of the class.
  *
  * Returns: zero on success, negative value on error.
  **/
