@@ -233,10 +233,10 @@ test_server_connect (void)
 	 * new connection has them automatically registered.
 	 */
 	TEST_FEATURE ("with existing jobs");
-	class1 = job_class_new (NULL, "foo");
+	class1 = job_class_new (NULL, "foo", NULL);
 	nih_hash_add (job_classes, &class1->entry);
 
-	class2 = job_class_new (NULL, "bar");
+	class2 = job_class_new (NULL, "bar", NULL);
 	job1 = job_new (class2, "test1");
 	job2 = job_new (class2, "test2");
 	nih_hash_add (job_classes, &class2->entry);
@@ -333,7 +333,6 @@ test_server_connect (void)
 	nih_free (class1);
 	nih_free (class2);
 
-
 	control_server_close ();
 
 	dbus_shutdown ();
@@ -534,10 +533,10 @@ test_bus_open (void)
 	refuse_registration = FALSE;
 	server_conn = NULL;
 
-	class1 = job_class_new (NULL, "foo");
+	class1 = job_class_new (NULL, "foo", NULL);
 	nih_hash_add (job_classes, &class1->entry);
 
-	class2 = job_class_new (NULL, "bar");
+	class2 = job_class_new (NULL, "bar", NULL);
 	job1 = job_new (class2, "test1");
 	job2 = job_new (class2, "test2");
 	nih_hash_add (job_classes, &class2->entry);
@@ -932,8 +931,11 @@ test_reload_configuration (void)
 
 	strcpy (filename, dirname);
 	strcat (filename, "/baz");
-	source3 = conf_source_new (NULL, filename, CONF_DIR);
 
+	/* XXX: note that this will generate an error message when this
+	 * test runs sine "/baz" does not exist as a directory.
+	 */
+	source3 = conf_source_new (NULL, filename, CONF_DIR);
 
 	message = nih_new (NULL, NihDBusMessage);
 	message->connection = NULL;
@@ -986,7 +988,7 @@ test_get_job_by_name (void)
 	nih_error_init ();
 	job_class_init ();
 
-	class = job_class_new (NULL, "test");
+	class = job_class_new (NULL, "test", NULL);
 	nih_hash_add (job_classes, &class->entry);
 
 
@@ -1101,13 +1103,13 @@ test_get_all_jobs (void)
 	 * in an array allocated as a child of the message structure.
 	 */
 	TEST_FEATURE ("with registered jobs");
-	class1 = job_class_new (NULL, "frodo");
+	class1 = job_class_new (NULL, "frodo", NULL);
 	nih_hash_add (job_classes, &class1->entry);
 
-	class2 = job_class_new (NULL, "bilbo");
+	class2 = job_class_new (NULL, "bilbo", NULL);
 	nih_hash_add (job_classes, &class2->entry);
 
-	class3 = job_class_new (NULL, "sauron");
+	class3 = job_class_new (NULL, "sauron", NULL);
 	nih_hash_add (job_classes, &class3->entry);
 
 	TEST_ALLOC_FAIL {
@@ -2217,6 +2219,9 @@ int
 main (int   argc,
       char *argv[])
 {
+	/* run tests in legacy (pre-session support) mode */
+	setenv ("UPSTART_NO_SESSIONS", "1", 1);
+
 	test_server_open ();
 	test_server_connect ();
 	test_server_close ();
