@@ -233,7 +233,7 @@ test_pending (void)
 		TEST_ALLOC_SAFE {
 			event = event_new (NULL, "test", NULL);
 
-			class = job_class_new (NULL, "test");
+			class = job_class_new (NULL, "test", NULL);
 			class->task = TRUE;
 			class->process[PROCESS_MAIN] = process_new (class->process);
 			class->process[PROCESS_MAIN]->command = "echo";
@@ -292,7 +292,7 @@ test_pending_handle_jobs (void)
 
 			TEST_FREE_TAG (event1);
 
-			class = job_class_new (NULL, "test");
+			class = job_class_new (NULL, "test", NULL);
 			class->task = TRUE;
 
 			class->start_on = event_operator_new (
@@ -327,7 +327,7 @@ test_pending_handle_jobs (void)
 
 			TEST_FREE_TAG (event1);
 
-			class = job_class_new (NULL, "test");
+			class = job_class_new (NULL, "test", NULL);
 			class->task = TRUE;
 
 			class->start_on = event_operator_new (
@@ -386,7 +386,7 @@ test_pending_handle_jobs (void)
 			TEST_FREE_TAG (event1);
 			TEST_FREE_TAG (event2);
 
-			class = job_class_new (NULL, "test");
+			class = job_class_new (NULL, "test", NULL);
 			class->task = TRUE;
 
 			assert (nih_str_array_add (&(class->env), class,
@@ -502,7 +502,7 @@ test_pending_handle_jobs (void)
 
 			TEST_FREE_TAG (event2);
 
-			class = job_class_new (NULL, "test");
+			class = job_class_new (NULL, "test", NULL);
 			class->task = TRUE;
 
 			assert (nih_str_array_add (&(class->env), class,
@@ -628,7 +628,7 @@ test_pending_handle_jobs (void)
 
 			TEST_FREE_TAG (event2);
 
-			class = job_class_new (NULL, "test");
+			class = job_class_new (NULL, "test", NULL);
 			class->task = TRUE;
 
 			assert (nih_str_array_add (&(class->env), class,
@@ -796,7 +796,7 @@ test_pending_handle_jobs (void)
 
 			TEST_FREE_TAG (event2);
 
-			class = job_class_new (NULL, "test");
+			class = job_class_new (NULL, "test", NULL);
 			class->task = TRUE;
 
 			assert (nih_str_array_add (&(class->env), class,
@@ -921,7 +921,7 @@ test_pending_handle_jobs (void)
 
 			TEST_FREE_TAG (event1);
 
-			class = job_class_new (NULL, "test");
+			class = job_class_new (NULL, "test", NULL);
 			class->instance = "$FRODO";
 			class->task = TRUE;
 
@@ -1000,7 +1000,7 @@ test_pending_handle_jobs (void)
 
 			TEST_FREE_TAG (event1);
 
-			class = job_class_new (NULL, "test");
+			class = job_class_new (NULL, "test", NULL);
 			class->instance = "$FRODO";
 			class->task = TRUE;
 
@@ -1084,7 +1084,7 @@ test_pending_handle_jobs (void)
 
 			TEST_FREE_TAG (event1);
 
-			class = job_class_new (NULL, "test");
+			class = job_class_new (NULL, "test", NULL);
 			class->instance = "$TIPPLE";
 			class->task = TRUE;
 
@@ -1127,7 +1127,7 @@ test_pending_handle_jobs (void)
 
 			TEST_FREE_TAG (event1);
 
-			class = job_class_new (NULL, "test");
+			class = job_class_new (NULL, "test", NULL);
 			class->task = TRUE;
 
 			class->process[PROCESS_POST_STOP] = process_new (class);
@@ -1175,7 +1175,7 @@ test_pending_handle_jobs (void)
 
 			TEST_FREE_TAG (event1);
 
-			class = job_class_new (NULL, "test");
+			class = job_class_new (NULL, "test", NULL);
 			class->task = TRUE;
 
 			class->process[PROCESS_POST_STOP] = process_new (class);
@@ -1250,7 +1250,7 @@ test_pending_handle_jobs (void)
 
 			TEST_FREE_TAG (event1);
 
-			class = job_class_new (NULL, "test");
+			class = job_class_new (NULL, "test", NULL);
 			class->task = TRUE;
 
 			class->process[PROCESS_POST_STOP] = process_new (class);
@@ -1329,7 +1329,7 @@ test_pending_handle_jobs (void)
 
 			TEST_FREE_TAG (event1);
 
-			class = job_class_new (NULL, "test");
+			class = job_class_new (NULL, "test", NULL);
 			class->task = TRUE;
 
 			class->process[PROCESS_POST_STOP] = process_new (class);
@@ -1439,7 +1439,7 @@ test_pending_handle_jobs (void)
 
 			TEST_FREE_TAG (event1);
 
-			class = job_class_new (NULL, "test");
+			class = job_class_new (NULL, "test", NULL);
 			class->task = TRUE;
 
 			class->process[PROCESS_POST_STOP] = process_new (class);
@@ -1527,7 +1527,7 @@ test_pending_handle_jobs (void)
 
 			TEST_FREE_TAG (event1);
 
-			class = job_class_new (NULL, "test");
+			class = job_class_new (NULL, "test", NULL);
 			class->task = TRUE;
 
 			class->process[PROCESS_POST_STOP] = process_new (class);
@@ -1623,7 +1623,7 @@ test_finished (void)
 
 			TEST_FREE_TAG (event);
 
-			class = job_class_new (NULL, "test");
+			class = job_class_new (NULL, "test", NULL);
 			class->process[PROCESS_MAIN] = process_new (class->process);
 			class->process[PROCESS_MAIN]->command = "echo";
 
@@ -1657,7 +1657,7 @@ test_finished (void)
 
 			TEST_FREE_TAG (event);
 
-			class = job_class_new (NULL, "test");
+			class = job_class_new (NULL, "test", NULL);
 			class->process[PROCESS_MAIN] = process_new (class->process);
 			class->process[PROCESS_MAIN]->command = "echo";
 
@@ -1699,7 +1699,7 @@ test_finished (void)
 
 			TEST_FREE_TAG (event);
 
-			class = job_class_new (NULL, "test");
+			class = job_class_new (NULL, "test", NULL);
 			class->process[PROCESS_MAIN] = process_new (class->process);
 			class->process[PROCESS_MAIN]->command = "echo";
 
@@ -1740,7 +1740,7 @@ test_finished (void)
 
 			TEST_FREE_TAG (event);
 
-			class = job_class_new (NULL, "foo");
+			class = job_class_new (NULL, "foo", NULL);
 
 			job = job_new (class, "");
 			job->goal = JOB_STOP;
@@ -1773,7 +1773,7 @@ test_finished (void)
 
 			TEST_FREE_TAG (event);
 
-			class = job_class_new (NULL, "foo");
+			class = job_class_new (NULL, "foo", NULL);
 
 			job = job_new (class, "");
 			job->goal = JOB_START;
@@ -1804,7 +1804,7 @@ test_finished (void)
 
 			TEST_FREE_TAG (event);
 
-			class = job_class_new (NULL, "foo");
+			class = job_class_new (NULL, "foo", NULL);
 
 			job = job_new (class, "");
 			job->goal = JOB_STOP;
@@ -1846,7 +1846,7 @@ test_finished (void)
 
 			TEST_FREE_TAG (event);
 
-			class = job_class_new (NULL, "foo");
+			class = job_class_new (NULL, "foo", NULL);
 
 			job = job_new (class, "");
 			job->goal = JOB_START;
@@ -1889,7 +1889,7 @@ test_finished (void)
 
 			TEST_FREE_TAG (event);
 
-			class = job_class_new (NULL, "foo");
+			class = job_class_new (NULL, "foo", NULL);
 			class->process[PROCESS_POST_STOP] = process_new (class);
 			class->process[PROCESS_POST_STOP]->command = "echo";
 
@@ -1934,7 +1934,7 @@ test_finished (void)
 
 			TEST_FREE_TAG (event);
 
-			class = job_class_new (NULL, "foo");
+			class = job_class_new (NULL, "foo", NULL);
 			class->process[PROCESS_PRE_START] = process_new (class);
 			class->process[PROCESS_PRE_START]->command = "echo";
 
@@ -1974,6 +1974,9 @@ int
 main (int   argc,
       char *argv[])
 {
+	/* run tests in legacy (pre-session support) mode */
+	setenv ("UPSTART_NO_SESSIONS", "1", 1);
+
 	test_new ();
 	test_block ();
 	test_unblock ();
